#include <PSX/Auxiliary/big_endian.hpp>
#include <PSX/File/file_processor_helper.hpp>
#include <stdio.hpp>

#ifdef __SUPPORT_VAG__
namespace JabyEngine {
    namespace FileProcessor {
        struct VAGHeader {
            char     id[4];
            uint32_t version;
            uint32_t reserved;
            uint32_t sample_size;
            uint32_t sample_frequency;
            uint8_t  reserved_2[12];
            char     name[16];

            constexpr uint32_t get_version() const {
                return read_be(this->version);
            }

            constexpr uint32_t get_sample_size() const {
                return read_be(this->sample_size);
            }

            constexpr uint32_t get_sample_frequency() const {
                return read_be(this->sample_frequency);
            }
        };

        struct VAGState {
            static constexpr VAGState create() {
                return VAGState{};
            }
        };

        static Progress parse_header(State::Configuration& config, VAGState& state) {
            if(config.data_bytes >= sizeof(VAGHeader)) {
                const auto& header = *reinterpret_cast<const VAGHeader*>(config.data_adr);

                //printf("VAG-ID: %s\nName: %s\nSample size: %i\nSample rate: %i\n", header.id, header.name, header.get_sample_size(), header.get_sample_frequency());
                // TODO: Allocate SPU memory
                return Progress::Error;
            }

            return Progress::InProgress;
        }

        State create(const uint32_t* data_adr, const VAG& file) {
            return State::from(VAGState::create(), data_adr, parse_header);
        }
    }
}
#endif // __SUPPORT_VAG__